<?php

/*

    recfriioの操作関数

*/


// 起動コマンドライン作成
// エラーの場合は FALSE を返す
function create_command( $operate, &$setting )
{
    show_debug( 'create_command : operate = '.$operate );

    if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ) $setting[ SET_ERRCODE ] |= ERR_BROKEN;

    // ローカルホスト
    $localhost = 'http://localhost';
    if( isset( $_SERVER['SERVER_PORT'] ) ) $localhost = $localhost.':'.$_SERVER['SERVER_PORT'];

    // do-record.phpまでのパス
    $serverpath = $_SERVER['PHP_SELF'];
    $serverpath = dirname( $serverpath ).'/do-record.php';

    // 時間
    $duration = $setting[ SET_DURATION ];
    if( $duration <= 0 ) $setting[ SET_ERRCODE ] |= ERR_DURATION;

    // スクリプトを起動してから sleep する時間
    $sleepsec = $setting[ SET_SLEEPSEC ];
    if( $sleepsec ) $duration += ( 60 - $sleepsec );

    // ファイル名
    if( $operate == OP_PLAY ){  // 再生のみの場合
        $outdir = '';
        $namebase = '/dev/null';
    }
    else{

        if( ! $setting[ SET_OUTDIR ] ) $setting[ SET_ERRCODE ] |= ERR_DIR;
        if( ! $setting[ SET_NAMEBASE ] ) $setting[ SET_ERRCODE ] |= ERR_NAME;

        // ディレクトリの最後が'/'で無い時は追加
        $outdir = $setting[ SET_OUTDIR ];
        if( $outdir && $outdir[ strlen( $outdir ) -1 ] != '/' ) $outdir = $outdir.'/';

        $namebase = $setting[ SET_NAMEBASE ];
    }

    // UDP送信
    if( $setting[ SET_UDP ] ){

        if( ! $setting[ SET_IP ] ) $setting[ SET_ERRCODE ] |= ERR_IP;
        if( ! $setting[ SET_PORT ] ) $setting[ SET_ERRCODE ] |= ERR_PORT;
    }

    if( $setting[ SET_ERRCODE ] ) return FALSE;
    if( ! $setting[ SET_ACTIVE ] ) return FALSE;

    // 実行スクリプトの前に ./ が付いていたら取り除いてフルパスにする
    $script = get_homedir().str_replace( './', '', RECORD_SCRIPT );

    $command = 

        'ID="'.$setting[ SET_ID ].'" '.

        'SERVERURL="'.$localhost.$serverpath.'" '.

        'CHANNEL="'.$setting[ SET_CHANNEL ].'" '.
        'DURATION="'.$duration.'" '.
        'UDP="'.$setting[ SET_UDP ].'" '.
        'IP="'.$setting[ SET_IP ].'" '.
        'PORT="'.$setting[ SET_PORT ].'" '.

        'OUTDIR="'.$outdir.'" '.
        'NAMEBASE="'.$namebase.'" '.
        'FILEEXT="'.FILEEXT.'" '.
        'OVERWRITE="'.$setting[ SET_OVERWRITE ].'" '.

        'RECFRIIO="'.RECFRIIO.'" '.
        'FRIIOOPT="'.FRIIOOPT.'" '.

        'WGET="'.WGET.'" '.
        'WGETOPT="'.WGETOPT.'" '.

        'SLEEPSEC="'.$sleepsec.'" '.

        $script
        ;

    show_debug( $command );

    return $command;
}


// 起動
function exec_recfriio( $operate, &$setting )
{
    if( $operate != OP_PLAY && $operate != OP_REC ) return FALSE;

    $setting[ SET_SLEEPSEC ] = 0;
    $command = create_command( $operate, $setting );
    if( ! $command ){

        $dummy = FALSE;
        throw new Exception( get_errmsg( $setting, $dummy ) );
    }

    exec( $command.' > /dev/null 2>&1 &' );
    sleep( WAITTIME );
}


// 停止
function stop_recfriio( $pid )
{
    show_debug( 'stop_recfriio pid = '.$pid );

    $command = KILLINTCMD.' '.$pid;
    show_debug( $command );

    exec( $command );
    sleep( WAITTIME );

    // 異常終了した時など、たまにステータスが消えずに残る時があるので削除しておく
    $statusfile = get_statusfile();
    if( file_exists( $statusfile ) ){
        read_csv( $statusfile, $status_array );
        remove_status( $status_array );
    }
}



// 週間予約設定のチューナへの割り当て情報を取得して $tuneritem_array に収録
function get_tuneritems_week( &$setting_array, &$tuneritem_array )
{
    foreach( $setting_array as $setting ){ 

        if( $setting[ SET_MODE ] != MODE_WEEK ) continue;
        if( ! $setting[ SET_ACTIVE ] ) continue;
        if( $setting[ SET_ERRCODE ] & ERR_TIME ) continue;
        if( $setting[ SET_ERRCODE ] & ERR_DATE ) continue;

        $hour = (int)$setting[ SET_HOUR ];
        $min = (int)$setting[ SET_MIN ];

        for( $week = 0; $week < 7; ++$week ){

            if( ! $setting[ SET_SUN + $week ] ) continue;

            $item = get_default_tuneritem();

            $item[ TN_ID ] = $setting[ SET_ID ];
            $item[ TN_START_W ] = $week * 24 * 3600 + $hour * 3600 + $min*60;
            $item[ TN_DURATION ] = $setting[ SET_DURATION ];
    
            $tuneritem_array[] = $item;
        }
    }

    if( _DEBUG ){
        show_debug( '-------------------' );
        show_debug( 'get_tuneritems_week' );
        foreach( $tuneritem_array as $item )
            show_debug( 'id='.$item[ TN_ID ].' start_week='.$item[ TN_START_W ].' dr='.$item[ TN_DURATION ] );
    }
}


// 一回予約設定のチューナへの割り当て情報を取得して $tuneritems_array に収録
//
// 週別に重複チェックを行うので、週別に割り当て情報を作成する
//
// $tuneritems_array[ 録画する週の日曜日の 0 時の UNIX時間 ][ 割り当て番号 ][ IN_ID ][ IN_START_W ][ TN_DURATION ]
//
function get_tuneritems_once( &$setting_array, &$tuneritems_array )
{
    foreach( $setting_array as $setting ){ 

        if( ! $setting[ SET_ACTIVE ] ) continue;
        if( $setting[ SET_MODE ] != MODE_ONCE ) continue;  // 週間予約は get_tuneritems_week() で取得して後で連結する
        if( $setting[ SET_ERRCODE ] & ERR_TIME ) continue;
        if( $setting[ SET_ERRCODE ] & ERR_DATE ) continue;

        $item = get_default_tuneritem();

        $nexttime = get_nexttime( $setting, time() );
        $year = (int)date( 'Y', $nexttime );
        $month = (int)date( 'm', $nexttime );
        $day = (int)date( 'd', $nexttime );
        $week = (int)date( 'w', $nexttime );
        $hour = (int)$setting[ SET_HOUR ];
        $min = (int)$setting[ SET_MIN ];

        // 録画する週の日曜日の 0 時の UNIX時間
        $start = mktime( 0, 0, 0, $month, $day, $year );
        $start -= 24 * 3600 * $week;

        $item[ TN_START_W ] = $week * 24 * 3600 + $hour * 3600 + $min*60;
        $item[ TN_DURATION ] = $setting[ SET_DURATION ];
        $item[ TN_ID ] = $setting[ SET_ID ];

        if( ! array_key_exists( $start, $tuneritems_array ) ) $tuneritems_array[ $start ] = array();
        $tuneritems_array[ $start ][] = $item;

        // もし土曜日で週をまたいでいたら次の週にも $item を追加
        $sec_week = 24 * 3600 * 7;
        if( $item[ TN_START_W ] + $item[ TN_DURATION ] >= $sec_week ){

            $start += $sec_week;
            $item[ TN_START_W ] -= $sec_week;

            if( ! array_key_exists( $start, $tuneritems_array ) ) $tuneritems_array[ $start ] = array();
            $tuneritems_array[ $start ][] = $item;
        }
    }

    // 週間予約分を追加
    $tuneritem_array_week = array();
    get_tuneritems_week( $setting_array, $tuneritem_array_week );
    if( count( $tuneritem_array_week ) ){

        if( ! count( $tuneritems_array ) ) $tuneritems_array[ 0 ] = array();

        foreach( $tuneritems_array as $st => $tmp ){

            if( ! count( $tuneritems_array[ $st ] ) ) $tuneritems_array[ $st ] = $tuneritem_array_week;
            else $tuneritems_array[ $st ] = array_merge( $tuneritems_array[ $st ], $tuneritem_array_week );
        }
    }

    // 週内での開始時刻でソート
    foreach( $tuneritems_array as $st => $tmp ){

        $start_w = array();
        foreach( $tuneritems_array[ $st ] as $item ) $start_w[] = $item[ TN_START_W ];
        array_multisort( $start_w, SORT_ASC, SORT_NUMERIC, $tuneritems_array[ $st ] );
    }

    if( _DEBUG ){

        show_debug( '-------------------' );
        show_debug( 'get_tuneritems_once' );
        foreach( $tuneritems_array as $st => $tuneritem_array ){

            show_debug( '> start='.$st );

            foreach( $tuneritem_array as $item ){
                show_debug( 'id='.$item[ TN_ID ].' start_week='.$item[ TN_START_W ].' dr='.$item[ TN_DURATION ] );
            }
        }
        show_debug( '-------------------' );
    }
}



// 予約時間の重複チェック
//
// 重複している予約IDの配列を返す
//
function check_duplicate( &$tuneritem_array )
{
    show_debug( '' );
    show_debug( 'check_duplicate' );

    $tuner_array = array();
    $dup_id_array = array();

    foreach( $tuneritem_array as $item ){

        $tmp_id_array = array();
        $tunerno = 0;

        while( TRUE ){

            $empty = TRUE;

            // まだチューナに一つも割当たっていないなら空いているチューナを探す
            if( ! ( count( $tuner_array ) <= $tunerno ) ){

                foreach( $tuner_array[ $tunerno ] as $target ){

                    $item_start = (int)$item[ TN_START_W ];
                    $item_end = (int)$item_start + (int)$item[ TN_DURATION ];

                    $target_start = (int)$target[ TN_START_W ];
                    $target_end = (int)$target_start + (int)$target[ TN_DURATION ];

                    $sec_week = 24 * 3600 * 7;
                    if( $target_end >= $sec_week && $item_end < $target_start ){

                        $item_start += $sec_week;
                        $item_end += $sec_week;
                    }
                    else if( $item_end >= $sec_week && $target_end < $item_start ){

                        $target_start += $sec_week;
                        $target_end += $sec_week;
                    }

                    // $targetと$itemの時間が重複している
                    if( $item_end > $target_start && $item_start < $target_end ){
    
                        show_debug( 'conflict tunerno='.$tunerno.' item='.$item[ TN_ID ].' with target='.$target[ TN_ID ] );

                        $tmp_id_array[] = $target[ TN_ID ];
                        $empty = FALSE;
                        break;
                    }
                }
            }

            // 空いているチューナがあった
            if( $empty ){

                $tuner_array[ $tunerno ][] = $item;
                break;
            }

            ++$tunerno;

            // 空いているチューナが無い
            if( $tunerno >= TUNERNUM ){

                show_debug( '!!! could not allocate tuner item='.$item[ TN_ID ] );

                $dup_id_array = array_merge( $dup_id_array, $tmp_id_array );
                $dup_id_array[] = $item[ TN_ID ];
                break;
            }
        }

        $tmp_id_array = array();
    }

    // チューナの割り当て状況を表示(デバッグ用)
    if( _DEBUG ){

        for( $i = 0; $i < count( $tuner_array ); ++$i ){

            show_debug( '----' );
            show_debug( 'tuner '.$i );
            foreach( $tuner_array[ $i ] as $item ){

                show_debug( 'id='.$item[ TN_ID ],
                            ' start_week='.$item[ TN_START_W ].
                            ' dr='.$item[ TN_DURATION ] );
            }
        }
        show_debug( '----' );

        foreach( $dup_id_array as $dup_id ) show_debug( 'dup_id ='.$dup_id );
    }

    return $dup_id_array;
}


// 予約設定の日付計算
function get_reservdate( &$setting )
{
    $date = '';
    $setting[ SET_ERRCODE ] = 0;

    if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ) $setting[ SET_ERRCODE ] |= ERR_BROKEN;

    $id = $setting[ SET_ID ];

    if( ( $setting[ SET_HOUR ] < 0 || $setting[ SET_HOUR ] > 23 )
        || ( $setting[ SET_MIN ] < 0  || $setting[ SET_MIN ] > 59 ) ) $setting[ SET_ERRCODE ] |= ERR_TIME;

    $digit = 0; 
    if( $setting[ SET_SLEEPSEC ] ) $digit = -1; // 1 分前にスクリプトを起動して $setting[ SET_SLEEPSEC ] 秒 sleep する
    $min = $setting[ SET_MIN ]  + $digit;  
    if( $min < 0 ){
        $min = 59;
        $digit = -1;
    }
    else $digit = 0;
       
    $hour = $setting[ SET_HOUR ] + $digit;
    if( $hour < 0 ){
        $hour = 23;
        $digit = -1;
    }
    else $digit = 0;

    $date = $min.' '.$hour;

    if( $setting[ SET_MODE ] != MODE_ONCE && $setting[ SET_MODE ] != MODE_WEEK ) $setting[ SET_ERRCODE ] |= ERR_MODE;
       
    if( $setting[ SET_MODE ] == MODE_ONCE ){

        if( ! check_date( $setting ) ) $setting[ SET_ERRCODE ] |= ERR_DATE;

        $month = (int) $setting[ SET_MONTH ];
        $day = (int) $setting[ SET_DAY ];

        // $digit == -1 の時は前日の日付にする
        if( $digit == -1 ){

            $time = get_nexttime( $setting, time() );
            $time -= 3600 * 24;
            $month = (int) date( 'm', $time );
            $day = (int) date( 'd', $time );
        }

        $date = $date.' '.$day.' '.$month;
    }
    else $date = $date.' * *';

    if( $setting[ SET_MODE ] == MODE_WEEK ){
        $week = '';
        if( $digit == 0 && $setting[ SET_SUN ] ) $week = $week.( ( 7 + $digit + 0 ) % 7 ).',';
        if( $setting[ SET_MON ] ) $week = $week.( ( 7 + $digit + 1 ) % 7 ).',';
        if( $setting[ SET_TUE ] ) $week = $week.( ( 7 + $digit + 2 ) % 7 ).',';
        if( $setting[ SET_WED ] ) $week = $week.( ( 7 + $digit + 3 ) % 7 ).',';
        if( $setting[ SET_THU ] ) $week = $week.( ( 7 + $digit + 4 ) % 7 ).',';
        if( $setting[ SET_FRI ] ) $week = $week.( ( 7 + $digit + 5 ) % 7 ).',';
        if( $setting[ SET_SAT ] ) $week = $week.( ( 7 + $digit + 6 ) % 7 ).',';
        if( $digit == -1 && $setting[ SET_SUN ] ) $week = $week.( ( 7 + $digit + 0 ) % 7 ).',';

        if( ! $week ) $setting[ SET_ERRCODE ] |= ERR_WEEK;

        $week = substr( $week, 0, strlen( $week )-1 );
        $date = $date.' '.$week;
    }
    else $date = $date.' *';

    if( $setting[ SET_SLEEPSEC ] <= 0 || $setting[ SET_SLEEPSEC ] >= 60  ) $setting[ SET_ERRCODE ] |= ERR_SLEEP;

    show_debug( 'date = '.$date );

    return $date;
}


// スクリプトをcrontab に登録
function regist_crontab( &$setting_array )
{
   // 予約設定が空の時はキャンセルする
   if( ! count( $setting_array ) ){

        file_put_contents( get_crontabfile(), '' );
        exec( 'crontab '.get_crontabfile() );

        return;
    }

    // 日付計算
    $date = array();
    for( $i = 0; $i < count( $setting_array ); ++$i ){

        $date[] = get_reservdate( $setting_array[ $i ] );
    }

    // 重複チェック
    $tuneritems_array = array();
    get_tuneritems_once( &$setting_array, &$tuneritems_array );
    foreach( $tuneritems_array as &$tuneritem_array ){

        $dup_id_array = check_duplicate( $tuneritem_array );
        if( count( $dup_id_array ) ){

            foreach( $dup_id_array as $dup_id ){

                foreach( $setting_array as &$setting_dup ){

                    if( $setting_dup[ SET_ID ] == $dup_id ){
        
                        $setting_dup[ SET_ERRCODE ] |= ERR_DUP;
                        break;
                    }
                }
            }
        }
    }

    // crontab のコマンド作成
    $crontab = '';
    for( $i = 0; $i < count( $setting_array ); ++$i ){

        $command = create_command( OP_REC, $setting_array[ $i ] );
        if( $command ) $crontab = $crontab.$date[ $i ].' '.$command."\n";
    }

    // crontab 登録
    if( ( $fp = fopen( get_crontabfile(), 'w' ) ) ) {

        fputs( $fp, $crontab );
        fclose( $fp );

        exec( 'crontab '.get_crontabfile() );
    }
    else throw new Exception(
        'エラー(regist_crontab) : crontab設定ファイル('.CRONTABFILE.')のオープンに失敗しました。パーミッションを確認してください。' );
}


// 動いていない recfriio の録画情報を消す
function remove_status( &$status_array )
{
    $lock = new Lock( FILELOCKDIR );

    $removed = FALSE;
    $flag = TRUE;

    while( $flag ){

        $flag = FALSE;
        for( $i = 0; $i < count( $status_array ); ++$i ){

            if( ! is_running( $status_array[ $i ][ PID ] ) ){

                array_splice( $status_array, $i, 1 );

                $flag = TRUE;
                $removed = TRUE;
                break;
            }
        }
    }

    if( $removed ){
        $statusfile = get_statusfile();
        if( ! count( $status_array ) && file_exists( $statusfile ) ){

            if( ! @unlink( $statusfile ) ){
                throw new Exception( 'エラー(remove_status) :ステータスファイル('.STATUSFILE.')の削除に失敗しました。パーミッションを確認してください。' );
            }
        }
        else save_csv( $statusfile, $status_array );
    }
}


// 現在録画中か
function is_recording( &$setting, $time )
{
    $duration = $setting[ SET_DURATION ];
    $sleepsec = $setting[ SET_SLEEPSEC ];
    if( $sleepsec ) $duration += ( 60 - $sleepsec );

    $nexttime = get_nexttime( $setting, $time - $duration );
    if( $nexttime <= $time && $time <= $nexttime + $duration ){

        $statusfile = get_statusfile();
        $status_array = array();
        if( file_exists( $statusfile ) ){

            read_csv( $statusfile, $status_array );

            foreach( $status_array as $status ) if( $status[ ID ] == $setting[ SET_ID ] ) return TRUE;
        }
    }

    return FALSE;
}


?>